// ***********************
// ** zx81keyboard_v2.h **
// ** Class Definitions **
// ** For ZX81 keyboard **
// ***********************

enum KEYMODES {EMULATOR, STANDARD};
enum KEYSTATES {NORMAL, NORMAL_SHIFTED, GRAPHICS, GRAPHICS_SHIFTED, FUNCTION};

#define NUM_ROWS 8
#define NUM_COLS 5

#define SHIFT_COL 4
#define SHIFT_ROW 5

//debounce countdown value, stop key from being depressed again by accident after release.
#define DEBOUNCE_DELAY 120
#define DEBOUNCE_DELAY_REPEAT 120

// Defines a single key and its 4 possible Major KEYSTATES: NORMAL, NORMAL_SHIFTED, GRAPHICS, GRAPHICS_SHIFTED
// Returns 5 KEYSTATES, adds FUNCTION: Caps Lock).
class ZxKey  {
	private:
	// Standard Keyboard Characters for Normal PC Use
	// Standard ZX keyboard only in lower case. EMULATOR KEYMODE should use byNormal only.
	byte byNormal;
	// Extra Modes for using keyboard as a normal(ish) PS2 / USB device
	byte byNormalShifted;	// Red Symbols & Words (replaced by Symbols)
	byte byGraphics;	// Standard ZX keyboard but should be used with CTL characters. Numbers = F1 - F10 etc
	byte byGraphicsShifted;	// Standard ZX keyboard but should be used with ALT characters. Numbers = F11 - F12, 5 = home, 6 PGUP etc
	// Monitor Keys last press / activity
	byte byDebounceCount;
	byte byKeyLastValue;
	boolean boKeyDisabled;

	byte KeyValue(KEYSTATES eKeystate) {
		// Get Keyboard character for correct Key state.
		byte byKeyValue = 0;

		switch (eKeystate) {
			case FUNCTION:
				byKeyValue = byNormal;
				// Adjust for Capital Letters
				if (byKeyValue > 96 && byKeyValue < 123){
					byKeyValue = byKeyValue - 32;
				}
				break;

			case NORMAL_SHIFTED:
				byKeyValue = byNormalShifted;
				break;

			case GRAPHICS:
				byKeyValue = byGraphics;
				break;

			case GRAPHICS_SHIFTED:
				byKeyValue = byGraphicsShifted;
				break;

			default:
				// Standard normal and Emulator mode keyboard.
				// Shift key modifier used later to select functions etc in a ZX81 Emulator.
				byKeyValue = byNormal;
				break;
		}

		return byKeyValue;
	}

	public:
	ZxKey (byte, byte, byte, byte);

		byte KeyPressDetected(KEYSTATES eKeystate, KEYMODES eKeymode){
			// Insure return of 0 if key not reset / debounced
			byte byKeyValue = 0;
			byDebounceCount--;
			if (byDebounceCount == 0){
				// Allow key to stay pressed until reset after release / debounce
				byDebounceCount = 1;
				if (boKeyDisabled == false){
					byKeyValue = KeyValue(eKeystate);
					byKeyLastValue = byKeyValue;
					boKeyDisabled = true;
				}
			}
			return byKeyValue;
		}

		byte KeyPressReset(){
			byte byKeyValue = 0;
			if (boKeyDisabled = true) {
				byDebounceCount = DEBOUNCE_DELAY;
				boKeyDisabled = false;
				byKeyValue = byKeyLastValue;
				byKeyLastValue = 0;
			}
			return byKeyValue;
		}

		void KeyDisable(){
			boKeyDisabled = true;
		}

};

// ZxKey constructor
ZxKey::ZxKey (byte Normal, byte NormalShifted, byte Graphics, byte GraphicsShifted){
	byNormal = Normal;
	byNormalShifted = NormalShifted;
	byGraphics = Graphics;
	byGraphicsShifted = GraphicsShifted;
	//Set starting debounce value
	byDebounceCount = DEBOUNCE_DELAY;
	byKeyLastValue = 0;
	boKeyDisabled = false;
}


// Defines entire keyboard, includes ZxKey class.
class ZxKeyBoard {
	private:

	// Setup 4 versions of keyboard states into keyboard.
	ZxKey keyMap[NUM_ROWS][NUM_COLS] =
	{
		{{'5',KEY_LEFT_ARROW,KEY_F5,KEY_HOME},{'4','%',KEY_F4,KEY_INSERT},{'3','#',KEY_F3,KEY_ESC},{'2','@',KEY_F2,KEY_F12},{'1','!',KEY_F1,KEY_F11}},
		{{'t','_','t','t'},{'r','&','r','r'},{'e','^','e','e'},{'w','`','w','w'},{'q','~','q','q'}},
		{{'6',KEY_DOWN_ARROW,KEY_F6,KEY_PAGE_DOWN},{'7',KEY_UP_ARROW,KEY_F7,KEY_PAGE_UP},{'8',KEY_RIGHT_ARROW,KEY_F8,KEY_END},{'9','9',KEY_F9,'9'},{'0',KEY_BACKSPACE,KEY_F10,'0'}},
		{{'g','\\','g','g'},{'f','}','f','f'},{'d','{','d','d'},{'s',']','s','s'},{'a','[','a','a'}},
		{{'y','|','y','y'},{'u','$','u','u'},{'i','(','i','i'},{'o',')','o','o'},{'p','"','p','p'}},
		{{'v','/','v','v'},{'c','?','c','c'},{'x',';','x','x'},{'z',':','z','z'},{0,0,0,0}},
		{{'h','\'','h','h'},{'j','-','j','j'},{'k','+','k','k'},{'l','=','l','l'},{KEY_RETURN,KEY_RETURN,KEY_RETURN,KEY_RETURN}},
		{{'b','*','b','b'},{'n','<','n','n'},{'m','>','m','m'},{'.',',','.','.'},{' ',KEY_TAB,'£',' '}}
	};

	public:

	// Return from ZxKey matching specified state.
	byte byKeyPress(byte byRow, byte byCol, KEYSTATES eKeystate, KEYMODES eKeymode){
		return keyMap[byRow][byCol].KeyPressDetected(eKeystate, eKeymode);
	}

	byte KeyPressReset(byte bRow, byte bCol){;
		return keyMap[bRow][bCol].KeyPressReset();
	}

	void KeyDisable(byte bRow, byte bCol){;
		//keyMap[bRow][bCol].KeyDisable();
	}
};


