// **************************************************************************
// **** ZX81 USB Keyboard for Funtronics LeoStick (based on a Leonardo). ****
// **** zx81usbkeyboard_v2.ino                                           ****
// **************************************************************************
// ** David Stephenson 2016-08-21  **
// ** Version 2                    **
// ** -----------------------------**
// ** Originally based on code by: **
// ** Dave Curran  2013-04-27      **
// ** Tony Smith 2014-02-15        **
// **********************************

#include "Arduino.h"
#include "zx81usbkeyboard_v2.h"
#include "zx81usbkeymodestate_v2.h"


// ************************
// *** Global Variables ***
// ************************

// Setup Global Variables for keyboard
ZxKeyBoard MyKeyboard;
// Setup LeoStick pins for mode switch and indicator LEDs
ModeState MyModeState(A5,A0,A1,A2);

// Setup LeoStick pins for Keyboard rows and columns
// Pins can be changed to work easily on a Leonarod of Pro-Micro
const byte byColPins[NUM_COLS] = {13,12,10,9,8};
const byte byRowPins[NUM_ROWS] = {7,6,5,4,3,2,1,0};

// Pin that piezo is connected to. should be 11 on a LeoStick
const byte byBeepPin = 11;

// Beep on keypress off by default. Turn on via serial port commands
// Beep always off in Emulator mode
// boDebEmu must be set to false and boKeyBeep must be true for beep to function
boolean boKeyBeep = false;

// Serial Port in, for reading commands
String sSerialIn = "";


KEYMODES eMyKeyMode;
KEYSTATES eMyKeyState;

// ******************
// *** Main Setup ***
// ******************
void setup() {

	// Set all Keyboard col pins as inputs and activate pull-ups
	for (byte byColCount = 0 ; byColCount < NUM_COLS ; byColCount++)
	{
		pinMode(byColPins[byColCount], INPUT);
		digitalWrite(byColPins[byColCount], HIGH);
	}

	// Set all Keyboard row pins as inputs
	for (byte byRowCount = 0 ; byRowCount < NUM_ROWS ; byRowCount++)
	{
		pinMode(byRowPins[byRowCount], INPUT);
	}

	// initialize control over the keyboard
	Keyboard.begin();
	// initialize serial port
	Serial.begin(115200);

}


// ************************
// *** Main loop        ***
// *** Lets get Busy-ah ***
//*************************
void loop() {

	bool boKeysShifted = false;
	byte byKeyPressed = 0;

	// Get Keyboard Mode
	eMyKeyMode = MyModeState.GetMode();
	eMyKeyState = MyModeState.GetState();

	// Check for the Shift key being pressed
	pinMode(byRowPins[SHIFT_ROW], OUTPUT);
	if (digitalRead(byColPins[SHIFT_COL]) == LOW) boKeysShifted = true;

	// Start General keyboard scan
	pinMode(byRowPins[SHIFT_ROW], INPUT);
	for (byte byRow = 0 ; byRow < NUM_ROWS ; byRow++){
		// Run through the rows, turn them on

		pinMode(byRowPins[byRow], OUTPUT);
		digitalWrite(byRowPins[byRow], LOW);

		for (byte byCol = 0 ; byCol < NUM_COLS ; byCol++){
			if (digitalRead(byColPins[byCol]) == LOW){

				if (boKeysShifted && eMyKeyMode == STANDARD){
					// Select correct Keyboard layout for current state For STANDARD mode
					// Shift alters the Current state selection. eg. gets Red Shift symbols in Normal State
					switch (eMyKeyState) {
						case NORMAL:
							byKeyPressed = MyKeyboard.byKeyPress(byRow, byCol, NORMAL_SHIFTED, eMyKeyMode);
							if (byKeyPressed == KEY_RETURN) {
								eMyKeyState = FUNCTION;
								byKeyPressed = 0;
								MyKeyboard.KeyDisable(byRow, byCol);
							}
							if (byKeyPressed == '9') {
								eMyKeyState = GRAPHICS;
								byKeyPressed = 0;
								MyKeyboard.KeyDisable(byRow, byCol);
							}
						break;

						case FUNCTION:
							byKeyPressed = MyKeyboard.byKeyPress(byRow, byCol, NORMAL_SHIFTED, eMyKeyMode);
							if (byKeyPressed == KEY_RETURN) {
								eMyKeyState = NORMAL;
								byKeyPressed = 0;
								MyKeyboard.KeyDisable(byRow, byCol);
							}
							if (byKeyPressed == '9') {
								eMyKeyState = GRAPHICS;
								byKeyPressed = 0;
								MyKeyboard.KeyDisable(byRow, byCol);
							}
						break;

						case GRAPHICS:
							byKeyPressed = MyKeyboard.byKeyPress(byRow, byCol, GRAPHICS_SHIFTED, eMyKeyMode);
							if (byKeyPressed == KEY_RETURN) {
								eMyKeyState = FUNCTION;
								byKeyPressed = 0;
								MyKeyboard.KeyDisable(byRow, byCol);
							}
							if (byKeyPressed == '9') {
								eMyKeyState = NORMAL;
								byKeyPressed = 0;
								MyKeyboard.KeyDisable(byRow, byCol);
							}
						break;

					}
				} else {
					byKeyPressed = MyKeyboard.byKeyPress(byRow, byCol, eMyKeyState, eMyKeyMode);
				}


				if (byKeyPressed > 0 ) {
					if (eMyKeyMode == EMULATOR){
						if (byKeyPressed && boKeysShifted) {
							SendKeyPress(byKeyPressed,KEY_LEFT_SHIFT);
						} else {
							SendKeyPress(byKeyPressed,0);
						}
					} else {
						if (eMyKeyState == GRAPHICS && byKeyPressed > 96 && byKeyPressed < 123){
							if (boKeysShifted){
								SendKeyPress(byKeyPressed,KEY_LEFT_ALT);
							} else {
								SendKeyPress(byKeyPressed,KEY_LEFT_CTRL);
							}
						} else {
							SendKeyPress(byKeyPressed,0);
						}
						// Some audio feedback if not in EMULATOR mode
                                                if (boKeyBeep == true){
                                                        if (boKeyBeep == true) tone(byBeepPin, 31, 20);
                                                }
					}
				}

			} else {
				byte byKeyValue = MyKeyboard.KeyPressReset(byRow, byCol);
				if (byKeyValue > 0){
					Keyboard.release(byKeyValue);
				}
			}
		}

		pinMode(byRowPins[byRow], INPUT);
	}


	// Reset for next scan
	digitalWrite(byRowPins[SHIFT_ROW], LOW);

	// Check for Commands / updates from USB Serial port
	if (Serial.available() > 0) commandIn();

	// Update LED panel and check for Mode change switch press
	MyModeState.SetState(eMyKeyState);
}


// Send Keypresses out to the world
void SendKeyPress(byte byKeyPressed, byte byKeyMod){
	if (byKeyMod > 0 ){
		Keyboard.press(byKeyMod);
		Keyboard.press(byKeyPressed);
		// need to use debounce for special keys, some emulators need some time to pickup keypress
		delay(DEBOUNCE_DELAY);
		Keyboard.release(byKeyMod);
	} else {
		Keyboard.press(byKeyPressed);
	}
}


// Read from serial port and ajust Mode and basic settings: Switch to EMU or Standards Mode etc
void commandIn(){

	//while (Serial.available()) {
	char inChar = (char)Serial.read();
	sSerialIn += inChar;
	sSerialIn.trim();
	sSerialIn.toUpperCase();

	String KEYWORDS[6] = {"DEVICE","EMULATOR","STANDARD","BELL","BEEPON","BEEPOFF"};

	if (sSerialIn.length() > 8) sSerialIn = sSerialIn.substring(1,9);
	for (byte bKeyword = 0 ; bKeyword < 6 ; bKeyword++){
	 if (sSerialIn.indexOf(KEYWORDS[bKeyword])!=-1){
		 if (KEYWORDS[bKeyword] == "DEVICE"){
			 Serial.println("ZX81_KEYS");
		 }
		 else if (KEYWORDS[bKeyword] == "BELL"){
			 tone(byBeepPin, 262, 60);
			 tone(byBeepPin, 196, 40);
		}

		else if (KEYWORDS[bKeyword] == "EMULATOR"){
			 MyModeState.SetMode(EMULATOR);
		}

		else if (KEYWORDS[bKeyword] == "STANDARD"){
			MyModeState.SetMode(STANDARD);
		}


		else if (KEYWORDS[bKeyword] == "BEEPON"){
			boKeyBeep = true;
		}

		else if (KEYWORDS[bKeyword] == "BEEPOFF"){
			 boKeyBeep = false;
		}

		// Clear Value if String found
		sSerialIn = "";
	 }
	}

}
